xquery version "1.0";
(: $Id: fpml-invariants-4-4-3.xq 154 2008-04-30 15:50:19Z u424971 $ :)

declare construction preserve;
declare copy-namespaces no-preserve, inherit;

declare default element namespace "http://www.fpml.org/2007/FpML-4-4";
import schema namespace fpml="http://www.fpml.org/2007/FpML-4-4" at "../../schema/fpml-main.xsd"; 
import schema namespace val="http://www.fpml.org/2007/FpML-4-4/validation" at "Validation.xsd";

declare function val:absent($arg as item()*) as xs:boolean
(: Return the not of exists :)
{
	not(exists($arg))
};

declare function val:iff($p as xs:boolean, $q as xs:boolean) as xs:boolean
(: Return the logical "if and only if" :)
{
	$p eq $q
};

declare function val:modulo-duration($dividendPeriodMultiplier as xs:integer, $dividendPeriod as xs:string, $divisorPeriodMultiplier as xs:integer, $divisorPeriod as xs:string) as xs:integer?
(: Return the mod of the duration in the smallest period supplied :)
{
	if ($dividendPeriod eq $divisorPeriod)
	then ($dividendPeriodMultiplier mod $divisorPeriodMultiplier)
	else if ($dividendPeriod eq "M" and $divisorPeriod eq "Y")
		then ($dividendPeriodMultiplier mod ($divisorPeriodMultiplier * 12))
		else if ($dividendPeriod eq "Y" and $divisorPeriod eq "M")
			then (($dividendPeriodMultiplier * 12) mod $divisorPeriodMultiplier)
			else if ($dividendPeriod eq "D" and $divisorPeriod eq "W")
				then ($dividendPeriodMultiplier mod ($divisorPeriodMultiplier * 7))
				else if ($dividendPeriod eq "W" and $divisorPeriod eq "D")
					then (($dividendPeriodMultiplier * 7) mod $divisorPeriodMultiplier * 12)
					else ()
};

declare function val:frequency-equivalence($dividendPeriodMultiplier as xs:integer, $dividendPeriod as xs:string, $divisorPeriodMultiplier as xs:integer, $divisorPeriod as xs:string) as xs:boolean?
(: Are the two frequencies known to be always equivalent :)
{
	if ($dividendPeriod eq $divisorPeriod)
	then ($dividendPeriodMultiplier eq $divisorPeriodMultiplier)
	else if ($dividendPeriod eq "M" and $divisorPeriod eq "Y")
		then ($dividendPeriodMultiplier eq ($divisorPeriodMultiplier * 12))
		else if ($dividendPeriod eq "Y" and $divisorPeriod eq "M")
			then (($dividendPeriodMultiplier * 12) eq $divisorPeriodMultiplier)
			else if ($dividendPeriod eq "D" and $divisorPeriod eq "W")
				then ($dividendPeriodMultiplier eq ($divisorPeriodMultiplier * 7))
				else if ($dividendPeriod eq "W" and $divisorPeriod eq "D")
					then (($dividendPeriodMultiplier * 7) eq $divisorPeriodMultiplier * 12)
					else (false())
};

declare function val:same-currency($money as element()*) as xs:boolean
(: Are all instances of a currency within these money elements are the same currency.
In Saxon-SA the signature would be declare function val:same-currency($money as fpml:Money()*) as xs:boolean  :)
{
	(count(distinct-values($money/currency)) le 1) and  (count(distinct-values($money/currency/@currencyScheme)) le 1)
};

declare function val:pretty-result($constraintName as xs:NMTOKEN, $satisfied as xs:boolean, $evaluationContext as element(), $caveat as xs:string?)
(: Pretty prints in XML the results of a rule evaluation. The context is only shown when the constraint is not satisfied :)
{
	validate strict {
		document {
			element val:contextEvaluation { 
				element val:constraint{$constraintName},
				element val:satisfied{$satisfied},
				element val:caveat{$caveat}[exists($caveat)],
				element val:context {$evaluationContext}[not($satisfied)] 
			}
		}
	}
};

let $creditIndexPreCondition as xs:boolean := fn:exists(//creditDefaultSwap/generalTerms/indexReferenceInformation) and not(exists(//creditDefaultSwap/generalTerms/indexReferenceInformation/tranche))

let $singleNamePreCondition as xs:boolean := exists(//creditDefaultSwap/generalTerms/referenceInformation)

let $sameCurrencyPreCondition as xs:boolean := count(distinct-values(//element(*, Money)/currency)) le 1

let $longFormPreCondition as xs:boolean := (count(//documentation/(masterConfirmation|contractualMatrix)) eq 0 ) and $singleNamePreCondition

let $shortFormPreCondition as xs:boolean := exists(//documentation[masterConfirmation|contractualMatrix|contractualTermsSupplement[matches (type, "^(iTraxx|CDX)")]])

let $iSDA2003PreCondition as xs:boolean := (some $contractualDefinitions in //documentation/contractualDefinitions satisfies starts-with($contractualDefinitions, "ISDA2003Credit")) or (matches(//documentation/masterConfirmation/masterConfirmationType, "^(ISDA2003Credit|ISDA2004Credit)"))

let $iSDA1999PreCondition as xs:boolean := some $document in (//element(*, Trade)|//element(*, Contract))/documentation/(contractualDefinitions|masterConfirmation/masterConfirmationType) satisfies contains($document, "ISDA1999Credit")

let $isParametric as xs:boolean := every $interestRateStream in //element(*, InterestRateStream) satisfies not(exists($interestRateStream/..//cashflows)) or ($interestRateStream/cashflows/cashflowsMatchParameters eq true()) 

let $documentEvaluation := validate strict{
	document {
		element val:documentEvaluation {

(for $trade in //element(*, Trade)[equityOption/equityExercise/equityAmericanExercise]
return val:pretty-result(xs:NMTOKEN("eqd-1"), $trade/equityOption/equityExercise/equityAmericanExercise/commencementDate/adjustableDate/unadjustedDate eq $trade/tradeHeader/tradeDate, $trade, ()))
,
(for $trade in //element(*, Trade)[equityOption/equityExercise/equityAmericanExercise]
return val:pretty-result(xs:NMTOKEN("eqd-2"), $trade/equityOption/equityExercise/equityAmericanExercise/expirationDate/adjustableDate/unadjustedDate ge $trade/tradeHeader/tradeDate, $trade, "See issue http://www.fpml.org/issues/view.php?id=672"))
, 
(for $trade in //element(*, Trade)[equityOption/equityExercise/equityBermudaExercise]
return val:pretty-result(xs:NMTOKEN("eqd-4"), $trade/equityOption/equityExercise/equityBermudaExercise/commencementDate/adjustableDate/unadjustedDate ge $trade//tradeHeader/tradeDate, $trade, "See issue http://www.fpml.org/issues/view.php?id=672"))
,
(for $trade in //element(*, Trade)[equityOption/equityExercise/equityBermudaExercise]
return val:pretty-result(xs:NMTOKEN("eqd-5"), $trade/equityOption/equityExercise/equityBermudaExercise/expirationDate/adjustableDate/unadjustedDate ge $trade/tradeHeader/tradeDate, $trade, "See issue http://www.fpml.org/issues/view.php?id=672"))
,
(for $trade in //element(*, Trade)[equityOption/equityExercise/equityEuropeanExercise]
return val:pretty-result(xs:NMTOKEN("eqd-12"), $trade/equityOption/equityExercise/equityEuropeanExercise/expirationDate/adjustableDate/unadjustedDate ge $trade/tradeHeader/tradeDate, $trade, "See issue http://www.fpml.org/issues/view.php?id=672"))
, 
(for $trade in //element(*, Trade)[equityOption/equityPremium/paymentDate/unadjustedDate]
return val:pretty-result(xs:NMTOKEN("eqd-13"), $trade/equityOption/equityPremium/paymentDate/unadjustedDate ge $trade/tradeHeader/tradeDate, $trade, "See issue http://www.fpml.org/issues/view.php?id=672"))
, 
(for $trade in //element(*, Trade)[brokerEquityOption/equityPremium/paymentDate/unadjustedDate]
return val:pretty-result(xs:NMTOKEN("eqd-14"), $trade/brokerEquityOption/equityPremium/paymentDate/unadjustedDate ge $trade/tradeHeader/tradeDate, $trade, "See issue http://www.fpml.org/issues/view.php?id=672"))
, 
(for $equityAmericanExercise in //element(*, EquityAmericanExercise)[latestExerciseTimeType="SpecificTime"]
return val:pretty-result(xs:NMTOKEN("eqd-3"), exists($equityAmericanExercise/latestExerciseTime), $equityAmericanExercise, "See issue http://www.fpml.org/issues/view.php?id=698"))
,
(for $equityBermudaExercise in //element(*, EquityBermudaExercise)[latestExerciseTimeType="SpecificTime"]
return val:pretty-result(xs:NMTOKEN("eqd-6"), exists($equityBermudaExercise/latestExerciseTime), $equityBermudaExercise, "See issue http://www.fpml.org/issues/view.php?id=699."))
,  
(for $equityBermudaExercise in //element(*, EquityBermudaExercise)
return val:pretty-result(xs:NMTOKEN("eqd-8"), every $date in $equityBermudaExercise/bermudaExerciseDates/date satisfies ($date gt $equityBermudaExercise/commencementDate/adjustableDate/unadjustedDate), $equityBermudaExercise, ()))
, 
(for $equityBermudaExercise in //element(*, EquityBermudaExercise)
return val:pretty-result(xs:NMTOKEN("eqd-9"), every $date in $equityBermudaExercise/bermudaExerciseDates/date satisfies $date le $equityBermudaExercise/expirationDate/adjustableDate/unadjustedDate, $equityBermudaExercise, ()))
,
(for $equityBermudaExercise in //element(*, EquityBermudaExercise)
return val:pretty-result(xs:NMTOKEN("eqd-10"), count(distinct-values($equityBermudaExercise/bermudaExerciseDates/date)) eq count($equityBermudaExercise/bermudaExerciseDates/date), $equityBermudaExercise, ()))
,
(for $equityExercise in //element(*, EquityExerciseValuationSettlement)[equityValuation/valuationDate/adjustableDate][equityEuropeanExercise]
return val:pretty-result(xs:NMTOKEN("eqd-15"), $equityExercise/equityValuation/valuationDate/adjustableDate/unadjustedDate eq $equityExercise/equityEuropeanExercise/expirationDate/adjustableDate/unadjustedDate, $equityExercise, "See issue http://www.fpml.org/issues/view.php?id=700."))
,
(for $equityOption in //element(*, EquityOption)[exists(equityExercise/*/element(*, EquityMultipleExercise))][exists(numberOfOptions)]
return val:pretty-result(xs:NMTOKEN("eqd-26"), $equityOption/equityExercise/*/element(*, EquityMultipleExercise)/maximumNumberOfOptions le $equityOption/numberOfOptions, $equityOption, "See issue: http://www.fpml.org/issues/view.php?id=679"))
, 
(for $equityMultipleExercise in //element(*, EquityMultipleExercise)
return val:pretty-result(xs:NMTOKEN("eqd-27"), $equityMultipleExercise/minimumNumberOfOptions le $equityMultipleExercise/maximumNumberOfOptions, $equityMultipleExercise, ()))
,
(for $equityMultipleExercise in //element(*, EquityMultipleExercise)[integralMultipleExercise]
return val:pretty-result(xs:NMTOKEN("eqd-28"), $equityMultipleExercise/minimumNumberOfOptions mod $equityMultipleExercise/integralMultipleExercise eq 0, $equityMultipleExercise, "See issue http://www.fpml.org/issues/view.php?id=701"))
, 
(for $equityMultipleExercise in //element(*, EquityMultipleExercise)[integralMultipleExercise]
return val:pretty-result(xs:NMTOKEN("eqd-29"), $equityMultipleExercise/maximumNumberOfOptions mod $equityMultipleExercise/integralMultipleExercise eq 0, $equityMultipleExercise, "See issue http://www.fpml.org/issues/view.php?id=702."))
, 
(for $equityOption in (//element(*, EquityOption)|//element(*, EquityDerivativeShortFormBase))[equityExercise/equityBermudaExercise/equityMultipleExercise][exists(numberOfOptions)]
return val:pretty-result(xs:NMTOKEN("eqd-18"), ($equityOption/equityExercise/equityBermudaExercise/equityMultipleExercise/maximumNumberOfOptions * $equityOption/equityExercise/equityBermudaExercise/equityMultipleExercise/integralMultipleExercise) le $equityOption/numberOfOptions, $equityOption, "See issue http://www.fpml.org/issues/view.php?id=677."))
, 
(for $equityOption in (//element(*, EquityOption)|//element(*, EquityDerivativeShortFormBase))[notional][equityPremium/percentageOfNotional][equityPremium/paymentAmount][val:same-currency((notional, equityPremium/paymentAmount))]
return val:pretty-result(xs:NMTOKEN("eqd-19"), $equityOption/equityPremium/paymentAmount/amount eq ($equityOption/notional/amount * $equityOption/equityPremium/percentageOfNotional), $equityOption, "See the issues: http://www.fpml.org/issues/view.php?id=683 and http://www.fpml.org/issues/view.php?id=677."))
,
(for $equityOption in (//element(*, EquityOption)|//element(*, EquityDerivativeShortFormBase))[numberOfOptions][optionEntitlement][equityPremium/paymentAmount][equityPremium/pricePerOption][val:same-currency((equityPremium/pricePerOption, equityPremium/paymentAmount))]
return val:pretty-result(xs:NMTOKEN("eqd-20"), ($equityOption/equityPremium/pricePerOption/amount * $equityOption/numberOfOptions * $equityOption/optionEntitlement = $equityOption/equityPremium/paymentAmount/amount), $equityOption, "See issue #684 http://www.fpml.org/issues/view.php?id=684"))
,
(for $equityDerivativeBase in //element(*, EquityDerivativeBase)
return val:pretty-result(xs:NMTOKEN("eqd-22"), $equityDerivativeBase/buyerPartyReference/@href ne $equityDerivativeBase/sellerPartyReference/@href, $equityDerivativeBase, ()))
,
(for $trade in (//element(*, Trade)|//element(*, Contract)|//element(*, Strategy))[equityOption/equityEffectiveDate]
return val:pretty-result(xs:NMTOKEN("eqd-23"), $trade/equityOption/equityEffectiveDate ge $trade/tradeHeader/tradeDate, $trade, "See issue: http://www.fpml.org/issues/view.php?id=685"))
,
(for $averagingSchedule in //element(*, AveragingSchedule)
return val:pretty-result(xs:NMTOKEN("eqd-24"), $averagingSchedule/startDate lt $averagingSchedule/endDate, $averagingSchedule, ()))
,
(for $brokerEquityOption in //element(*, BrokerEquityOption)[exists(numberOfOptions)][exists(equityPremium/paymentAmount)][exists(equityPremium/pricePerOption)][val:same-currency((equityPremium/paymentAmount,equityPremium/pricePerOption))]
return val:pretty-result(xs:NMTOKEN("eqd-25"), $brokerEquityOption/equityPremium/pricePerOption/amount * $brokerEquityOption/numberOfOptions eq $brokerEquityOption/equityPremium/paymentAmount/amount, $brokerEquityOption, "See issue: http://www.fpml.org/issues/view.php?id=686"))
,
(for $businessDayAdjustments as element(*, BusinessDayAdjustments) in //element(*, BusinessDayAdjustments) 
let $businessCentersPresent as xs:boolean := exists($businessDayAdjustments[businessCentersReference|businessCenters]) 
let $businessDayConventionNotSet as xs:boolean := not($businessDayAdjustments/businessDayConvention = ('NONE', 'NotApplicable')) 
return val:pretty-result(xs:NMTOKEN("shared-1"), $businessCentersPresent eq $businessDayConventionNotSet, $businessDayAdjustments, ())) 
, 
(for $relativeDateOffset as element(*, RelativeDateOffset) in //element(*, RelativeDateOffset)[dayType = 'Business'] 
return val:pretty-result(xs:NMTOKEN("shared-4"), $relativeDateOffset/businessDayConvention eq 'NONE', $relativeDateOffset, ())) 
, 
(for $payerPartyReference as element(payerPartyReference) in //payerPartyReference[../receiverPartyReference] 
return val:pretty-result(xs:NMTOKEN("shared-5"), $payerPartyReference/../receiverPartyReference/@href ne $payerPartyReference/@href, $payerPartyReference, ())) 
, 
(for $americanExercise as element(*, AmericanExercise) in //element(*, AmericanExercise)[latestExerciseTime] 
return val:pretty-result(xs:NMTOKEN("shared-6"), $americanExercise/earliestExerciseTime/hourMinuteTime lt $americanExercise/latestExerciseTime/hourMinuteTime, $americanExercise, ())) 
, 
(for $bermudaExercise as element(*, BermudaExercise) in //element(*, BermudaExercise)[latestExerciseTime] 
return val:pretty-result(xs:NMTOKEN("shared-7"), $bermudaExercise/earliestExerciseTime/hourMinuteTime lt $bermudaExercise/latestExerciseTime/hourMinuteTime, $bermudaExercise, ())) 
, 
(for $dateRange as element(*, DateRange) in //element(*, DateRange) 
return val:pretty-result(xs:NMTOKEN("shared-8"), $dateRange/unadjustedFirstDate lt $dateRange/unadjustedLastDate, $dateRange, ())) 
, 
(for $dateRange as element(*, BusinessDateRange) in //element(*, BusinessDateRange) 
let $businessCentersPresent as xs:boolean := exists($dateRange[businessCentersReference|businessCenters]) 
let $businessDayConventionNotSet as xs:boolean := not($dateRange/businessDayConvention = ('NONE', 'NotApplicable')) 
return val:pretty-result(xs:NMTOKEN("shared-9"), $businessCentersPresent eq $businessDayConventionNotSet, $dateRange, ())) 
, 
(for $calculationAgent as element(calculationAgent) in //calculationAgent
return val:pretty-result(xs:NMTOKEN("shared-10"), count(distinct-values($calculationAgent/calculationAgentPartyReference/@href)) eq count($calculationAgent/calculationAgentPartyReference/@href), $calculationAgent, ())) 
, 
(for $trade as element(*, Trade) in //element(*, Trade)[//businessDateRange]
return val:pretty-result(xs:NMTOKEN("shared-11"), every $href in $trade//businessCentersReference/@href satisfies $href = $trade//businessCenters/@id, $trade, ())) 
, 
(for $fpML as element(FpML) in //FpML
return val:pretty-result(xs:NMTOKEN("shared-12"), every $href in $fpML/buyerPartyReference/@href satisfies $href = $fpML/(party|tradeSide)/@id, $fpML, ())) 
, 
(for $fpML as element(FpML) in //FpML
return val:pretty-result(xs:NMTOKEN("shared-13"), every $href in $fpML/sellerPartyReference/@href satisfies $href = $fpML/(party|tradeSide)/@id, $fpML, ())) 
,
(for $fpML as element(FpML) in //FpML
return val:pretty-result(xs:NMTOKEN("shared-14"), every $href in $fpML/calculationAgentPartyReference/@href satisfies $href = $fpML/party/@id, $fpML, ())) 
,
(for $offset as element(*, Offset) in //element(*, Offset) 
let $dayTypePresent as xs:boolean := exists($offset/dayType) 
return val:pretty-result(xs:NMTOKEN("shared-15"), $dayTypePresent eq ($offset/period eq 'D' and $offset/periodMultiplier ne 0), $offset, ())) 
,
(for $tradeSide as element(*, TradeSide) in //element(*, TradeSide)
return val:pretty-result(xs:NMTOKEN("shared-16"), every $href in $tradeSide//party/@href satisfies $href = /FpML/party/@id, $tradeSide, ())) 
, 
(for $tradeSide as element(*, TradeSide) in //element(*, TradeSide)
return val:pretty-result(xs:NMTOKEN("shared-17"), every $href in $tradeSide//account/@href satisfies $href = /FpML/party/account/@id, $tradeSide, ())) 
, 
(for $interestRateStream as element(*, InterestRateStream) in //element(*, InterestRateStream)[exists(calculationPeriodAmount/calculation/(floatingRateCalculation|inflationRateCalculation))]
return val:pretty-result(xs:NMTOKEN("ird-1"), exists($interestRateStream/resetDates), $interestRateStream, ()))
, 
(for $interestRateStream as element(*, InterestRateStream) in //element(*, InterestRateStream)[$isParametric]
return val:pretty-result(xs:NMTOKEN("ird-2"), val:modulo-duration($interestRateStream/paymentDates/paymentFrequency/periodMultiplier, $interestRateStream/paymentDates/paymentFrequency/period, $interestRateStream/calculationPeriodDates/calculationPeriodFrequency/periodMultiplier, $interestRateStream/calculationPeriodDates/calculationPeriodFrequency/period)=0, $interestRateStream, ()))
,
(: to be implemented
TODO unadjusted calculation period dates is complex to work out!

ird-3 (Mandatory)
Preconditions: isParametric
paymentDates/firstPaymentDate must match one of the unadjusted calculation period dates.

ird-4 (Mandatory)
Preconditions: isParametric
paymentDates/lastRegularPaymentDate must match one of the unadjusted calculation period dates
:)

(: The FpML rule is bust. It should take into account whether it is a resetting swapStream or not because $interestRateStream/resetDates is optional. An issue is raised for this: http://www.fpml.org/issues/view.php?id=613
(for $interestRateStream as element(*, InterestRateStream) in //element(*, InterestRateStream)[$isParametric]
return <result constraint="ird-5" valid="{val:modulo-duration($interestRateStream/calculationPeriodDates/calculationPeriodFrequency/periodMultiplier, $interestRateStream/calculationPeriodDates/calculationPeriodFrequency/period, $interestRateStream/resetDates/resetFrequency/periodMultiplier, $interestRateStream/resetDates/resetFrequency/period)=0}"></result>)

ird-5 (Mandatory)
Preconditions: isParametric
The frequency in calculationPeriodDates/calculationPeriodFrequencymust be an integer multiple of the frequency in resetDates/resetFrequency. See also: frequency equivalence
:)

(: This FpML rule is bust. Two issues are raised against it: http://www.fpml.org/issues/view.php?id=614 http://www.fpml.org/issues/view.php?id=615.
(for $interestRateStream as element(*, InterestRateStream) in //element(*, InterestRateStream)[$isParametric]
return <result constraint="ird-6" valid="{$interestRateStream/paymentDates/firstPaymentDate > $interestRateStream/calculationPeriodDates/effectiveDate/unadjustedDate}"></result>)
ird-6 (Mandatory)
Preconditions: isParametric
paymentDates/firstPaymentDate > calculationPeriodDates/effectiveDate/unadjustedDate.
:)
(for $interestRateStream as element(*, InterestRateStream) in //element(*, InterestRateStream)[$isParametric]
return val:pretty-result(xs:NMTOKEN("ird-7"), val:iff(exists($interestRateStream/calculationPeriodAmount/calculation/compoundingMethod), not (val:frequency-equivalence($interestRateStream/paymentDates/paymentFrequency/periodMultiplier, $interestRateStream/paymentDates/paymentFrequency/period, $interestRateStream/calculationPeriodDates/calculationPeriodFrequency/periodMultiplier, $interestRateStream/calculationPeriodDates/calculationPeriodFrequency/period))), $interestRateStream, ()))
,
(for $interestRateStream as element(*, InterestRateStream) in //element(*, InterestRateStream)
return val:pretty-result(xs:NMTOKEN("ird-8"), $interestRateStream/payerPartyReference/@href ne $interestRateStream/receiverPartyReference/@href, $interestRateStream, ()))
,
(for $interestRateStream as element(*, InterestRateStream) in //element(*, InterestRateStream)
return val:pretty-result(xs:NMTOKEN("ird-9"), if (exists($interestRateStream/calculationPeriodAmount/calculation/compoundingMethod)) then exists($interestRateStream/resetDates) else true(), $interestRateStream, ()))


(: IRD Constraints to be implemented. All of these constraints are broken in some way. Most are not yet based on types.

ird-10 (Mandatory)
Preconditions: isParametric
This rule applies when calculationPeriodFrequency/rollConvention is a number. Let the start date be firstRegularPeriodStartDate if it exists, or effectiveDate/unadjustedDate otherwise: If calculationPeriodFrequency/rollConvention is less than or equal to the last day of the month in the start date, then the day of the start date must match it. Otherwise the day of the start date must be the last day of the month of that date.
Test cases: [Valid] [Valid] [Valid] [Valid] [Valid] [Invalid] [Invalid] [Invalid] [Invalid]
ird-11 (Mandatory)
Preconditions: isParametric
This rule applies when calculationPeriodFrequency/rollConvention is a number. Let the end date be lastRegularPeriodEndDate if it exists, or terminationDate/unadjustedDate otherwise: If calculationPeriodFrequency/rollConvention is less than or equal to the last day of the month in the end date, then the day of the end date must match it. Otherwise the day of the end date must be the last day of the month of that date.
Test cases: [Valid] [Valid] [Valid] [Valid] [Valid] [Invalid] [Invalid] [Invalid] [Invalid] [Invalid]
ird-12 (Mandatory)
Preconditions: isParametric
The frequency specified in calculationPeriodFrequency must divide the regular period precisely. This means that by stepping through the period from the start date at the specified frequency, it must be possible to reach the end date.
Test cases: [Valid] [Valid] [Valid] [Valid] [Invalid] [Invalid] [Invalid] [Invalid] [Invalid] [Invalid]
ird-14 (Mandatory)
Preconditions: isParametric
terminationDate/unadjustedDate > effectiveDate/unadjustedDate.
Test cases: [Valid] [Invalid] [Invalid]
ird-15 (Mandatory)
Preconditions: isParametric
terminationDate/unadjustedDate > firstPeriodStartDate/unadjustedDate.
Test cases: [Valid] [Invalid] [Invalid]
ird-16 (Mandatory)
Preconditions: isParametric
terminationDate/unadjustedDate > firstRegularPeriodStartDate.
Test cases: [Valid] [Invalid] [Invalid]
ird-17 (Mandatory)
Preconditions: isParametric
terminationDate/unadjustedDate > lastRegularPeriodEndDate.
Test cases: [Valid] [Invalid]
ird-18 (Mandatory)
Preconditions: isParametric
lastRegularPeriodEndDate > firstRegularPeriodStartDate.
Test cases: [Valid] [Invalid] [Invalid]
ird-19 (Mandatory)
Preconditions: isParametric
lastRegularPeriodEndDate > firstPeriodStartDate/unadjustedDate.
Test cases: [Valid] [Invalid] [Invalid]
ird-20 (Mandatory)
Preconditions: isParametric
lastRegularPeriodEndDate > effectiveDate/unadjustedDate
Test cases: [Valid] [Invalid]
ird-21 (Mandatory)
Preconditions: isParametric
firstPeriodStartDate/unadjustedDate < effectiveDate/unadjustedDate
Test cases: [Valid] [Invalid] [Invalid]
ird-22 (Mandatory)
Preconditions: isParametric
firstPeriodStartDate/unadjustedDate < firstRegularPeriodStartDate
Test cases: [Valid] [Invalid] [Invalid]
ird-57 (Mandatory)
Preconditions: isParametric
If rollConvention is neither NONE nor SFE, nor a day of the week ( MON, TUE, WED, THU, FRI, SAT or SUN) then the period must be M or Y.
Test cases: [Valid] [Valid] [Valid] [Valid] [Invalid]
ird-58 (Mandatory)
Preconditions: isParametric
If rollConvention is a day of the week ( MON, TUE, WED, THU, FRI, SAT or SUN) then the period must be W.
Test cases: [Valid] [Valid] [Valid] [Valid] [Invalid]
Context: stubCalculationPeriodAmount
ird-23 (Mandatory)
initialStub should only be present if the calculationPeriodDates element referenced by calculationPeriodDates/@href contains at least one of:

    * firstPeriodStartDate
    * firstRegularPeriodStartDate

Test cases: [Valid] [Valid] [Invalid]
ird-24 (Mandatory)
finalStub should only be present if the calculationPeriodDates element referenced by calculationPeriodDates/@href contains a lastRegularPeriodEndDate element.
Test cases: [Valid] [Invalid]
Context: Schedule (complex type)
ird-25 (Mandatory)
If there are no step elements then initialValue must be non-zero.
Test cases: [Valid] [Valid] [Invalid]
Context: businessCentersReference
ird-26 (Mandatory)
The @href attribute must match the @id attribute of a businessCenters element somewhere within the document.
Test cases: [Valid] [Valid] [Invalid] [Invalid]
Context: MandatoryEarlyTermination (complex type)
ird-27 (Mandatory)
cashSettlement/cashSettlementPaymentDate must not be present.
Test cases: [Valid] [Invalid]
ird-28 (Mandatory)
In cashSettlement/cashSettlementValuationDate/dateRelativeTo, the @href attribute must point to the @id attribute of mandatoryEarlyTerminationDate.
Test cases: [Valid] [Invalid]
Context: Calculation (complex type)
ird-29 (Mandatory)
If compoundingMethod is present then fixedRateSchedule must not be present.
Test cases: [Valid] [Invalid]
Context: CalculationPeriod (complex type)
ird-30 (Mandatory)
Must contain unadjustedStartDate and/or adjustedStartDate.
Test cases: [Valid] [Valid] [Invalid]
ird-31 (Mandatory)
Must contain unadjustedEndDate and/or adjustedEndDate.
Test cases: [Valid] [Valid] [Invalid]



Context: notionalStepSchedule
ird-50 (Mandatory)
Preconditions: isParametric
The dates in step/stepDate must be unadjusted calculation period dates in ../../../../calculationPeriodDates.
Test cases: [Valid] [Valid] [Invalid] [Invalid] [Invalid]
Context: fixedRateSchedule
ird-51 (Mandatory)
Preconditions: isParametric
The dates in step/stepDate must be unadjusted calculation period dates in ../../../calculationPeriodDates.
Test cases: [Valid] [Valid] [Invalid] [Invalid] [Invalid] [Invalid]
Context: capRateSchedule
ird-52 (Mandatory)
Preconditions: isParametric
The dates in step/stepDate must be unadjusted calculation period dates in ../../../../calculationPeriodDates.
Test cases: [Valid] [Valid] [Invalid] [Invalid] [Invalid] [Invalid]
Context: floorRateSchedule
ird-53 (Mandatory)
Preconditions: isParametric
The dates in step/stepDate must be unadjusted calculation period dates in ../../../../calculationPeriodDates.
Test cases: [Valid] [Valid] [Invalid] [Invalid] [Invalid] [Invalid]
Context: knownAmountSchedule
ird-54 (Mandatory)
Preconditions: isParametric
The dates in step/stepDate must be unadjusted calculation period dates in ../../calculationPeriodDates.
Test cases: [Valid] [Valid] [Invalid] [Invalid] [Invalid] [Invalid]
Context: paymentDates
ird-55 (Mandatory)
If calculationPeriodDatesReference exists, the @href attribute must point to the @id attribute of calculationPeriodDates in the same swapStream.
Test cases: [Valid] [Invalid]
ird-56 (Mandatory)
If resetDatesReference exists, the @href attribute must point to the @id attribute of resetDates in the same swapStream.
Test cases: [Valid] [Invalid]
Context: resetDates
ird-59 (Mandatory)
In calculationPeriodDatesReference, the @href attribute must point to the @id attribute of calculationPeriodDates in the same swapStream.


Context: PaymentDates (complex type)
ird-36 (Mandatory)
This rule has the problem that paymentFrequency duration is  a different type to the difference between the dates.
If both firstPaymentDate and lastRegularPaymentDate are present then the period defined by these dates must be an integer multiple of paymentFrequency.
Test cases: [Valid] [Invalid]


:)
,
(for $discounting as element(*, Discounting) in //element(*, Discounting)
return val:pretty-result(xs:NMTOKEN("ird-32"), if (val:absent($discounting/discountRate)) then (val:absent($discounting/discountRateDayCountFraction)) else true(), $discounting, ("The example invalid-ird-32-00.xml is broken. It is not schema valid. Please see the issues:  http://www.fpml.org/issues/view.php?id=650"))) 
,
(for $fra as element(*, Fra) in //element(*, Fra)
return val:pretty-result(xs:NMTOKEN("ird-33"), $fra/adjustedTerminationDate gt $fra/adjustedEffectiveDate, $fra, ())) 
,
(for $paymentCalculationPeriod as element(*, PaymentCalculationPeriod) in //element(*, PaymentCalculationPeriod)
return val:pretty-result(xs:NMTOKEN("ird-34"), exists($paymentCalculationPeriod/(unadjustedPaymentDate|adjustedPaymentDate)), $paymentCalculationPeriod, ())) 
,
(for $paymentDates as element(*, PaymentDates) in //element(*, PaymentDates)[exists(firstPaymentDate)][exists(lastRegularPaymentDate)]
return val:pretty-result(xs:NMTOKEN("ird-35"), xs:boolean($paymentDates/firstPaymentDate lt $paymentDates/lastRegularPaymentDate), $paymentDates, ())) 
,
(for $stubCalculationPeriodAmount as element(*, StubCalculationPeriodAmount) in //element(*, StubCalculationPeriodAmount)
return val:pretty-result(xs:NMTOKEN("ird-38"), exists($stubCalculationPeriodAmount/(initialStub|finalStub)), $stubCalculationPeriodAmount, ())) 
,
(for $earlyTerminationEvent as element(*, EarlyTerminationEvent) in //element(*, EarlyTerminationEvent)
return val:pretty-result(xs:NMTOKEN("ird-39"), $earlyTerminationEvent/adjustedExerciseDate le $earlyTerminationEvent/adjustedEarlyTerminationDate, $earlyTerminationEvent, ())) 
,
(for $earlyTerminationEvent as element(*, EarlyTerminationEvent) in //element(*, EarlyTerminationEvent)
return val:pretty-result(xs:NMTOKEN("ird-40"), $earlyTerminationEvent/adjustedExerciseDate le $earlyTerminationEvent/adjustedCashSettlementValuationDate, $earlyTerminationEvent, ())) 
,
(for $earlyTerminationEvent as element(*, EarlyTerminationEvent) in //element(*, EarlyTerminationEvent)
return val:pretty-result(xs:NMTOKEN("ird-41"), $earlyTerminationEvent/adjustedCashSettlementValuationDate le $earlyTerminationEvent/adjustedCashSettlementPaymentDate, $earlyTerminationEvent, ())) 
,
(for $extensionEvent as element(*, ExtensionEvent) in //element(*, ExtensionEvent)
return val:pretty-result(xs:NMTOKEN("ird-42"), $extensionEvent/adjustedExerciseDate lt $extensionEvent/adjustedExtendedTerminationDate, $extensionEvent, ())) 
,
(for $fxLinkedNotionalAmount as element(*, FxLinkedNotionalAmount) in //element(*, FxLinkedNotionalAmount)
return val:pretty-result(xs:NMTOKEN("ird-43"), exists($fxLinkedNotionalAmount/element()), $fxLinkedNotionalAmount, ())) 
,
(for $mandatoryEarlyTerminationAdjustedDates as element(*, MandatoryEarlyTerminationAdjustedDates) in //element(*, MandatoryEarlyTerminationAdjustedDates)
return val:pretty-result(xs:NMTOKEN("ird-44"), ($mandatoryEarlyTerminationAdjustedDates/adjustedEarlyTerminationDate le $mandatoryEarlyTerminationAdjustedDates/adjustedCashSettlementValuationDate) and ($mandatoryEarlyTerminationAdjustedDates/adjustedCashSettlementValuationDate le $mandatoryEarlyTerminationAdjustedDates/adjustedCashSettlementPaymentDate), $mandatoryEarlyTerminationAdjustedDates, ())) 
,
(for $optionalEarlyTermination as element(*, OptionalEarlyTermination) in //element(*, OptionalEarlyTermination)[cashSettlement/cashSettlementValuationDate]
return val:pretty-result(xs:NMTOKEN("ird-46"), (if (exists($optionalEarlyTermination/cashSettlement/cashSettlementPaymentDate))  then ($optionalEarlyTermination/cashSettlement/cashSettlementValuationDate/dateRelativeTo/@href eq $optionalEarlyTermination/cashSettlement/cashSettlementPaymentDate/@id) else false()), $optionalEarlyTermination, ())) 
,
(for $optionalEarlyTermination as element(*, OptionalEarlyTermination) in //element(*, OptionalEarlyTermination)[exists(cashSettlement/cashSettlementPaymentDate/relativeDate)]
return val:pretty-result(xs:NMTOKEN("ird-47"), $optionalEarlyTermination/cashSettlement/cashSettlementPaymentDate/relativeDate/dateRelativeTo/@href eq $optionalEarlyTermination/(americanExercise|bermudaExercise|europeanExercise)/@id, $optionalEarlyTermination, ("See issue: http://www.fpml.org/issues/view.php?id=647"))) 
,
(for $swaption as element(*, Swaption) in //element(*, Swaption)[exists(cashSettlement/cashSettlementPaymentDate)]
return val:pretty-result(xs:NMTOKEN("ird-48"), $swaption/cashSettlement/cashSettlementPaymentDate/relativeDate/dateRelativeTo/@href eq $swaption/(americanExercise|bermudaExercise|europeanExercise)/@id, $swaption, ())) 
,
(for $resetFrequency as element(*, ResetFrequency) in //element(*, ResetFrequency)
return val:pretty-result(xs:NMTOKEN("ird-49"), val:iff(exists($resetFrequency/weeklyRollConvention), ($resetFrequency/period eq "W")), $resetFrequency, ())) 
,
(for $trade as element(*, Trade) in //element(*, Trade)[$singleNamePreCondition][exists(creditDefaultSwap)]
return val:pretty-result(xs:NMTOKEN("cd-1"), $trade/tradeHeader/tradeDate lt $trade/creditDefaultSwap/generalTerms/effectiveDate/unadjustedDate, $trade, ("See issue: http://www.fpml.org/issues/view.php?id=616"))) 
, 
(for $trade as element(*, Trade) in //element(*, Trade)[$creditIndexPreCondition][exists(creditDefaultSwap/generalTerms/effectiveDate)]
return val:pretty-result(xs:NMTOKEN("cd-1b"), $trade/tradeHeader/tradeDate ge $trade/creditDefaultSwap/generalTerms/effectiveDate/unadjustedDate, $trade, ("http://www.fpml.org/issues/view.php?id=708"))) 
, 
(for $trade as element(*, Trade) in //element(*, Trade)
return val:pretty-result(xs:NMTOKEN("cd-2"),  every $calculationAgent in $trade/calculationAgent satisfies (($calculationAgent/calculationAgentParty = "AsSpecifiedInMasterAgreement") or $calculationAgent/calculationAgentPartyReference), $trade, ())) 
,  
(for $contractualTermsSupplement as element(*, ContractualTermsSupplement) in //element(*, ContractualTermsSupplement)[$iSDA1999PreCondition]
return val:pretty-result(xs:NMTOKEN("cd-3b"), xs:boolean(not(starts-with($contractualTermsSupplement/type, "ISDA2003Credit"))), $contractualTermsSupplement, "See issue: http://www.fpml.org/issues/view.php?id=709, http://www.fpml.org/issues/view.php?id=710")) 
, 
(for $trade as element(*, Trade) in //element(*, Trade)[$iSDA2003PreCondition]
return val:pretty-result(xs:NMTOKEN("cd-4b"), every $contractualSupplement in $trade/documentation/contractualSupplement satisfies not(starts-with($contractualSupplement, "ISDA1999Credit")), $trade, ())) 
,
(for $generalTerms as element(*, GeneralTerms) in //element(*, GeneralTerms)[exists(effectiveDate)][exists(scheduledTerminationDate)]
return val:pretty-result(xs:NMTOKEN("cd-5"), $generalTerms/effectiveDate/unadjustedDate lt $generalTerms/scheduledTerminationDate/adjustableDate/unadjustedDate, $generalTerms, ())) 
,
(for $generalTerms as element(*, GeneralTerms) in //element(*, GeneralTerms)
return val:pretty-result(xs:NMTOKEN("cd-6"), $generalTerms/buyerPartyReference/@href ne $generalTerms/sellerPartyReference/@href, $generalTerms, ())) 
,
(for $generalTerms as element(*, GeneralTerms) in //element(*, GeneralTerms)[$longFormPreCondition]
return val:pretty-result(xs:NMTOKEN("cd-7"), exists($generalTerms/effectiveDate/dateAdjustments) or exists($generalTerms/effectiveDate/dateAdjustmentsReference), $generalTerms, ())) 
,
(for $generalTerms as element(*, GeneralTerms) in //element(*, GeneralTerms)[$longFormPreCondition]
return val:pretty-result(xs:NMTOKEN("cd-8"), exists($generalTerms/effectiveDate/dateAdjustments) or exists($generalTerms/effectiveDate/dateAdjustmentsReference), $generalTerms, ())) 
,
(for $generalTerms as element(*, GeneralTerms) in //element(*, GeneralTerms)
return val:pretty-result(xs:NMTOKEN("cd-40"), if (not(exists($generalTerms/indexReferenceInformation/tranche))) then not(exists($generalTerms/modifiedEquityDelivery)) else true(), $generalTerms, ())) 
,
(for $generalTerms as element(*, GeneralTerms) in //element(*, GeneralTerms)
return val:pretty-result(xs:NMTOKEN("cd-42"), if (not(exists($generalTerms/basketReferenceInformation))) then not(exists($generalTerms/substitution)) else true(), $generalTerms, ())) 
,
(for $referenceInformation as element(*, ReferenceInformation) in //element(*, ReferenceInformation)
return val:pretty-result(xs:NMTOKEN("cd-9"), if (exists($referenceInformation/referenceObligation/primaryObligorReference)) then (every $referenceObligation in $referenceInformation/referenceObligation satisfies $referenceObligation/primaryObligorReference/@href eq $referenceInformation/referenceEntity/@id) else true(), $referenceInformation, ())) 
,
(for $referenceInformation as element(*, ReferenceInformation) in //element(*, ReferenceInformation)
return val:pretty-result(xs:NMTOKEN("cd-10"), if (exists($referenceInformation/referenceObligation/guarantorReference)) then ($referenceInformation/referenceObligation/guarantorReference/@href eq $referenceInformation/referenceEntity/@id) else true(), $referenceInformation, ())) 
,
(for $referenceInformation as element(*, ReferenceInformation) in //element(*, ReferenceInformation)[$longFormPreCondition][$iSDA2003PreCondition]
return val:pretty-result(xs:NMTOKEN("cd-11"), exists($referenceInformation/allGuarantees), $referenceInformation, ())) 
,
(for $referenceInformation as element(*, ReferenceInformation) in //element(*, ReferenceInformation)
return val:pretty-result(xs:NMTOKEN("cd-12"), if (exists($referenceInformation/referencePrice)) then ($referenceInformation/referencePrice ge 0) else true(), $referenceInformation, ())) 
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)
return val:pretty-result(xs:NMTOKEN("cd-13"), if (exists($creditDefaultSwap/protectionTerms/creditEvents/creditEventNotice/notifyingParty/buyerPartyReference)) then ($creditDefaultSwap/protectionTerms/creditEvents/creditEventNotice/notifyingParty/buyerPartyReference/@href eq $creditDefaultSwap/generalTerms/buyerPartyReference/@href) else true(), $creditDefaultSwap, ())) 
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)
return val:pretty-result(xs:NMTOKEN("cd-14"), if (exists($creditDefaultSwap/protectionTerms/creditEvents/creditEventNotice/notifyingParty/sellerPartyReference)) then ($creditDefaultSwap/protectionTerms/creditEvents/creditEventNotice/notifyingParty/sellerPartyReference/@href eq $creditDefaultSwap/generalTerms/sellerPartyReference/@href) else true(), $creditDefaultSwap, ())) 
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)
return val:pretty-result(xs:NMTOKEN("cd-15"), if ((count($creditDefaultSwap/generalTerms/referenceInformation/referenceObligation) eq 1) and exists($creditDefaultSwap/cashSettlementTerms/valuationDate/singleValuationDate)) then $creditDefaultSwap/cashSettlementTerms/valuationMethod = ("Market", "Highest") else true(), $creditDefaultSwap, ())) 
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)
return val:pretty-result(xs:NMTOKEN("cd-16"), if ((count($creditDefaultSwap/generalTerms/referenceInformation/referenceObligation) eq 1) and exists($creditDefaultSwap/cashSettlementTerms/valuationDate/multipleValuationDates)) then $creditDefaultSwap/cashSettlementTerms/valuationMethod = ("AverageMarket", "Highest", "AverageHighestMarket") else true(), $creditDefaultSwap, ())) 
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)
return val:pretty-result(xs:NMTOKEN("cd-17"), if ((count($creditDefaultSwap/generalTerms/referenceInformation/referenceObligation) gt 1) and exists($creditDefaultSwap/cashSettlementTerms/valuationDate/singleValuationDate)) then $creditDefaultSwap/cashSettlementTerms/valuationMethod = ("BlendedMarket", "BlendedHighest") else true(), $creditDefaultSwap, ())) 
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)
return val:pretty-result(xs:NMTOKEN("cd-18"), if ((count($creditDefaultSwap/generalTerms/referenceInformation/referenceObligation) gt 1) and exists($creditDefaultSwap/cashSettlementTerms/valuationDate/multipleValuationDates)) then $creditDefaultSwap/cashSettlementTerms/valuationMethod = ("AverageBlendedMarket", "AverageBlendedHighest") else true(), $creditDefaultSwap, ())) 
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)[$iSDA1999PreCondition]
return val:pretty-result(xs:NMTOKEN("cd-19"), not(exists($creditDefaultSwap/protectionTerms/creditEvents/creditEventNotice/businessCenter) or exists($creditDefaultSwap/protectionTerms/creditEvents/restructuring/multipleHolderObligation) or exists($creditDefaultSwap/protectionTerms/creditEvents/restructuring/multipleCreditEventNotices) or exists($creditDefaultSwap/generalTerms/referenceInformation/allGuarantees) or exists($creditDefaultSwap/generalTerms/indexReferenceInformation) or exists($creditDefaultSwap/generalTerms/substitution) or exists($creditDefaultSwap/generalTerms/modifiedEquityDelivery)), $creditDefaultSwap, ()))
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)[$iSDA2003PreCondition]
return val:pretty-result(xs:NMTOKEN("cd-20"), not(exists($creditDefaultSwap/protectionTerms/obligations/notContingent)), $creditDefaultSwap, ()))
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)[$shortFormPreCondition][$singleNamePreCondition]
return val:pretty-result(xs:NMTOKEN("cd-21"), not(exists($creditDefaultSwap/cashSettlementTerms) 
	or exists($creditDefaultSwap/physicalSettlementTerms) 
	or exists($creditDefaultSwap/feeLeg/periodicPayment/fixedAmountCalculation/calculationAmount) 
	or exists($creditDefaultSwap/feeLeg/periodicPayment/fixedAmountCalculation/dayCountFraction) 
	or exists($creditDefaultSwap/protectionTerms/obligations) 
	or exists($creditDefaultSwap/generalTerms/referenceInformation/allGuarantees) 
	or exists($creditDefaultSwap/generalTerms/referenceInformation/referencePrice) 
	or exists($creditDefaultSwap/generalTerms/effectiveDate/dateAdjustments) 
	or exists($creditDefaultSwap/generalTerms/effectiveDate/dateAdjustmentsReference) 
	or exists($creditDefaultSwap/generalTerms/scheduledTerminationDate/adjustableDate/dateAdjustments) 
	or exists($creditDefaultSwap/generalTerms/scheduledTerminationDate/adjustableDate/dateAdjustmentsReference) 
	or exists($creditDefaultSwap/generalTerms/dateAdjustments)), $creditDefaultSwap, ()))
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)[$shortFormPreCondition][$creditIndexPreCondition]
return val:pretty-result(xs:NMTOKEN("cd-21b"), not(exists($creditDefaultSwap/cashSettlementTerms) 
	or exists($creditDefaultSwap/physicalSettlementTerms) 
	or exists($creditDefaultSwap/feeLeg/periodicPayment/fixedAmountCalculation/calculationAmount) 
	or exists($creditDefaultSwap/feeLeg/periodicPayment/fixedAmountCalculation/dayCountFraction) 
	or exists($creditDefaultSwap/protectionTerms/obligations) 
	or exists($creditDefaultSwap/generalTerms/effectiveDate/dateAdjustments) 
	or exists($creditDefaultSwap/generalTerms/effectiveDate/dateAdjustmentsReference) 
	or exists($creditDefaultSwap/generalTerms/scheduledTerminationDate/adjustableDate/dateAdjustments) 
	or exists($creditDefaultSwap/generalTerms/scheduledTerminationDate/adjustableDate/dateAdjustmentsReference) 
	or exists($creditDefaultSwap/generalTerms/dateAdjustments) 
	or exists($creditDefaultSwap/protectionTerms/creditEvents)), $creditDefaultSwap, ()))
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)[$shortFormPreCondition]
return val:pretty-result(xs:NMTOKEN("cd-22"), not(exists($creditDefaultSwap/protectionTerms/creditEvents/bankruptcy) 
	or exists($creditDefaultSwap/protectionTerms/creditEvents/failureToPay) 
	or exists($creditDefaultSwap/protectionTerms/creditEvents/repudiationMoratorium) 
	or exists($creditDefaultSwap/protectionTerms/creditEvents/obligationDefault) 
	or exists($creditDefaultSwap/protectionTerms/creditEvents/obligationAcceleration)), $creditDefaultSwap, ()))
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)[$longFormPreCondition]
return val:pretty-result(xs:NMTOKEN("cd-23"), exists($creditDefaultSwap/cashSettlementTerms) or exists($creditDefaultSwap/physicalSettlementTerms), $creditDefaultSwap, ()))
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)[$longFormPreCondition]
return val:pretty-result(xs:NMTOKEN("cd-24"), exists($creditDefaultSwap/protectionTerms/creditEvents) and exists($creditDefaultSwap/protectionTerms/creditEvents/creditEventNotice) and exists($creditDefaultSwap/protectionTerms/obligations) and exists($creditDefaultSwap/generalTerms/referenceInformation/referencePrice), $creditDefaultSwap, ()))
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)[$longFormPreCondition]
return val:pretty-result(xs:NMTOKEN("cd-25"), if (exists($creditDefaultSwap/physicalSettlementTerms)) then (exists($creditDefaultSwap/physicalSettlementTerms/settlementCurrency) and exists($creditDefaultSwap/physicalSettlementTerms/physicalSettlementPeriod) and exists($creditDefaultSwap/physicalSettlementTerms/escrow) and exists($creditDefaultSwap/physicalSettlementTerms/deliverableObligations/accruedInterest)) else true(), $creditDefaultSwap, ()))
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)
return val:pretty-result(xs:NMTOKEN("cd-26"), if (exists($creditDefaultSwap/feeLeg/singlePayment/adjustablePaymentDate)) then ($creditDefaultSwap/feeLeg/singlePayment/adjustablePaymentDate gt $creditDefaultSwap/generalTerms/effectiveDate/unadjustedDate) else true(), $creditDefaultSwap, ()))
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)
return val:pretty-result(xs:NMTOKEN("cd-27"), if (exists($creditDefaultSwap/feeLeg/singlePayment/adjustablePaymentDate) and exists($creditDefaultSwap/generalTerms/scheduledTerminationDate/adjustableDate/unadjustedDate)) then ($creditDefaultSwap/feeLeg/singlePayment/adjustablePaymentDate lt $creditDefaultSwap/generalTerms/scheduledTerminationDate/adjustableDate/unadjustedDate) else true(), $creditDefaultSwap, ()))
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)
return val:pretty-result(xs:NMTOKEN("cd-28"), if (exists($creditDefaultSwap/feeLeg/periodicPayment/firstPaymentDate)) then ($creditDefaultSwap/feeLeg/periodicPayment/firstPaymentDate gt $creditDefaultSwap/generalTerms/effectiveDate/unadjustedDate) else true(), $creditDefaultSwap, ()))
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)
return val:pretty-result(xs:NMTOKEN("cd-29"), if (exists($creditDefaultSwap/feeLeg/periodicPayment/firstPaymentDate) and exists($creditDefaultSwap/generalTerms/scheduledTerminationDate/adjustableDate)) then ($creditDefaultSwap/feeLeg/periodicPayment/firstPaymentDate lt $creditDefaultSwap/generalTerms/scheduledTerminationDate/adjustableDate/unadjustedDate) else true(), $creditDefaultSwap, ()))
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)
return val:pretty-result(xs:NMTOKEN("cd-30"), if (exists($creditDefaultSwap/feeLeg/periodicPayment/lastRegularPaymentDate) and exists($creditDefaultSwap/generalTerms/scheduledTerminationDate/adjustableDate)) then ($creditDefaultSwap/feeLeg/periodicPayment/lastRegularPaymentDate lt $creditDefaultSwap/generalTerms/scheduledTerminationDate/adjustableDate/unadjustedDate) else true(), $creditDefaultSwap, ()))
,
(for $creditDefaultSwap as element(*, CreditDefaultSwap) in //element(*, CreditDefaultSwap)[$singleNamePreCondition]
return val:pretty-result(xs:NMTOKEN("cd-43"), if (exists($creditDefaultSwap/feeLeg/initialPayment)) then (($creditDefaultSwap/feeLeg/initialPayment/payerPartyReference/@href eq 
$creditDefaultSwap/generalTerms/sellerPartyReference/@href) and ($creditDefaultSwap/feeLeg/initialPayment/receiverPartyReference/@href eq $creditDefaultSwap/generalTerms/buyerPartyReference/@href)) else true(), $creditDefaultSwap, ()))
,
(for $periodicPayment as element(*, PeriodicPayment) in //element(*, PeriodicPayment)
return val:pretty-result(xs:NMTOKEN("cd-31"), if (exists($periodicPayment/firstPaymentDate) and exists($periodicPayment/lastRegularPaymentDate)) then ($periodicPayment/firstPaymentDate lt $periodicPayment/lastRegularPaymentDate) else true(), $periodicPayment, ()))
,
(for $periodicPayment as element(*, PeriodicPayment) in //element(*, PeriodicPayment)[$longFormPreCondition]
return val:pretty-result(xs:NMTOKEN("cd-31"), if (exists($periodicPayment/fixedAmountCalculation) and exists($periodicPayment/lastRegularPaymentDate)) then ($periodicPayment/firstPaymentDate lt $periodicPayment/lastRegularPaymentDate) else true(), $periodicPayment, ()))
,
(for $periodicPayment as element(*, PeriodicPayment) in //element(*, PeriodicPayment)[$longFormPreCondition]
return val:pretty-result(xs:NMTOKEN("cd-32"), if (exists($periodicPayment/fixedAmountCalculation)) then (exists($periodicPayment/fixedAmountCalculation/calculationAmount) and exists($periodicPayment/fixedAmountCalculation/dayCountFraction)) else true(), $periodicPayment, ()))
(:
Rule cd-33 is badly broken. See issue: http://www.fpml.org/issues/view.php?id=636
cd-33 (Mandatory)
If both firstPaymentDate and lastRegularPaymentDate are present, then lastRegularPaymentDate must fall precisely on a date reachable by adding an integer multiple of the period in paymentFrequency to firstPaymentDate.
Test cases: [Valid] [Invalid] [Invalid]
:)
,
(for $deliverableObligations as element(*, DeliverableObligations) in //element(*, DeliverableObligations)
return val:pretty-result(xs:NMTOKEN("cd-34"), if ($deliverableObligations/category eq "ReferenceObligationsOnly") then (count($deliverableObligations/element()) = 1) else true(), $deliverableObligations, ()))
,
(for $creditEvents as element(*, CreditEvents) in //element(*, CreditEvents)
return val:pretty-result(xs:NMTOKEN("cd-35"), count($creditEvents/element()) ge 1, $creditEvents, ()))
,
(for $publiclyAvailableInformation as element(*, PubliclyAvailableInformation) in //element(*, PubliclyAvailableInformation)
return val:pretty-result(xs:NMTOKEN("cd-36"), exists($publiclyAvailableInformation/standardPublicSources) or exists($publiclyAvailableInformation/publicSource), $publiclyAvailableInformation, ()))
,
(for $cashSettlementTerms as element(*, CashSettlementTerms) in //element(*, CashSettlementTerms)[$sameCurrencyPreCondition][exists(quotationAmount)][exists(minimumQuotationAmount)]
return val:pretty-result(xs:NMTOKEN("cd-37"), $cashSettlementTerms/quotationAmount/amount ge $cashSettlementTerms/minimumQuotationAmount/amount, $cashSettlementTerms, ()))
,
(for $referencePool as element(*, ReferencePool) in //element(*, ReferencePool)
return val:pretty-result(xs:NMTOKEN("cd-38"), (sum($referencePool/referencePoolItem/constituentWeight/basketPercentage) eq 1) or not(exists($referencePool/referencePoolItem/constituentWeight)), $referencePool, ("See the issue: http://www.fpml.org/issues/view.php?id=644"))) 
,
(for $basketReferenceInformation as element(*, BasketReferenceInformation) in //element(*, BasketReferenceInformation)
return val:pretty-result(xs:NMTOKEN("cd-39"), if (exists($basketReferenceInformation/nthToDefault) and exists($basketReferenceInformation/mthToDefault)) then ($basketReferenceInformation/nthToDefault lt $basketReferenceInformation/mthToDefault) else true(), $basketReferenceInformation, ()))
,
(for $tranche as element(*, Tranche) in //element(*, Tranche)
return val:pretty-result(xs:NMTOKEN("cd-39"), $tranche/attachmentPoint le $tranche/exhaustionPoint, $tranche, ()))

		}	
	}
}

return (
	validate strict{
		document {
			element val:validation {
				attribute xsi:schemaLocation {"http://www.fpml.org/2007/FpML-4-4/validation Validation.xsd"},
				element val:allSatisfied {every $satisfied in $documentEvaluation/val:contextEvaluation/val:satisfied satisfies $satisfied eq true()},
				$documentEvaluation
			}	
		}
	}
)
